@@ -50,6 +50,7 @@ static struct {
 
 #define _alloc(x) pool_zalloc(_cache.mem, (x))
 #define _free(x) pool_free(_cache.mem, (x))
+#define _strdup(x) pool_strdup(_cache.mem, (x))
 
 static int _insert(const char *path, int rec);
 
@@ -69,7 +70,12 @@ struct device *dev_create_file(const char *filename, struct device *dev,
 				dbg_free(dev);
 				return NULL;
 			}
-
+			if (!(alias->str = dbg_strdup(filename))) {
+				log_error("filename strdup failed");
+				dbg_free(dev);
+				dbg_free(alias);
+				return NULL;
+			}
 			dev->flags = DEV_ALLOCED;
 		} else {
 			if (!(dev = _alloc(sizeof(*dev)))) {
@@ -81,15 +87,13 @@ struct device *dev_create_file(const char *filename, struct device *dev,
 				_free(dev);
 				return NULL;
 			}
+			if (!(alias->str = _strdup(filename))) {
+				log_error("filename strdup failed");
+				return NULL;
+			}
 		}
-	}
-
-	if (!(alias->str = dbg_strdup(filename))) {
+	} else if (!(alias->str = dbg_strdup(filename))) {
 		log_error("filename strdup failed");
-		if (allocate && use_malloc) {
-			dbg_free(dev);
-			dbg_free(alias);
-		}
 		return NULL;
 	}
 
@@ -192,7 +196,7 @@ static int _compare_paths(const char *path0, const char *path1)
 static int _add_alias(struct device *dev, const char *path)
 {
 	struct str_list *sl = _alloc(sizeof(*sl));
-	struct list *ah;
+	struct str_list *strl;
 	const char *oldpath;
 	int prefer_old = 1;
 
@@ -202,8 +206,8 @@ static int _add_alias(struct device *dev, const char *path)
 	}
 
 	/* Is name already there? */
-	list_iterate(ah, &dev->aliases) {
-		if (!strcmp(list_item(ah, struct str_list)->str, path)) {
+	list_iterate_items(strl, &dev->aliases) {
+		if (!strcmp(strl->str, path)) {
 			log_debug("%s: Already in device cache", path);
 			return 1;
 		}
@@ -410,20 +414,16 @@ static int _insert(const char *path, int rec)
 
 static void _full_scan(int dev_scan)
 {
-	struct list *dh;
+	struct dir_list *dl;
 
 	if (_cache.has_scanned && !dev_scan)
 		return;
 
-	list_iterate(dh, &_cache.dirs) {
-		struct dir_list *dl = list_item(dh, struct dir_list);
+	list_iterate_items(dl, &_cache.dirs)
 		_insert_dir(dl->dir);
-	};
 
-	list_iterate(dh, &_cache.files) {
-		struct dir_list *dl = list_item(dh, struct dir_list);
+	list_iterate_items(dl, &_cache.files)
 		_insert_file(dl->dir);
-	};
 
 	_cache.has_scanned = 1;
 	init_full_scan_done(1);
