@@ -85,6 +85,7 @@ arg(size_ARG, 'L', "size", size_mb_arg)
 arg(logicalextent_ARG, 'L', "logicalextent", int_arg_with_sign)
 arg(persistent_ARG, 'M', "persistent", yes_no_arg)
 arg(major_ARG, 'j', "major", major_arg)
+arg(mirrors_ARG, 'm', "mirrors", int_arg)
 arg(metadatatype_ARG, 'M', "metadatatype", metadatatype_arg)
 arg(maps_ARG, 'm', "maps", NULL)
 arg(name_ARG, 'n', "name", string_arg)
@@ -100,6 +101,7 @@ arg(physicalvolume_ARG, 'P', "physicalvolume", NULL)
 arg(readahead_ARG, 'r', "readahead", int_arg)
 arg(resizefs_ARG, 'r', "resizefs", NULL)
 arg(reset_ARG, 'R', "reset", NULL)
+arg(regionsize_ARG, 'R', "regionsize", size_mb_arg)
 arg(physicalextentsize_ARG, 's', "physicalextentsize", size_mb_arg)
 arg(stdin_ARG, 's', "stdin", NULL)
 arg(snapshot_ARG, 's', "snapshot", NULL)
