@@ -86,13 +86,13 @@ static int _remove_lv(struct cmd_context *cmd, struct logical_volume *lv,
 		cow = snap_seg->cow;
 
 		*list_unsafe = 1;	/* May remove caller's lvht! */
-		if (!vg_remove_snapshot(lv->vg, cow)) {
+		if (!vg_remove_snapshot(cow)) {
 			stack;
 			return 0;
 		}
 		log_verbose("Removing LV %s from VG %s", cow->name,
 			    lv->vg->name);
-		if (!lv_remove(lv->vg, cow)) {
+		if (!lv_remove(cow)) {
 			stack;
 			return 0;
 		}
@@ -100,7 +100,7 @@ static int _remove_lv(struct cmd_context *cmd, struct logical_volume *lv,
 
 	/* Remove the LV itself */
 	log_verbose("Removing LV %s from VG %s", lv->name, lv->vg->name);
-	if (!lv_remove(lv->vg, lv)) {
+	if (!lv_remove(lv)) {
 		stack;
 		return 0;
 	}
@@ -129,12 +129,12 @@ static int _make_vg_consistent(struct cmd_context *cmd, struct volume_group *vg)
 		/* Are any segments of this LV on missing PVs? */
 		list_iterate_items(seg, &lv->segments) {
 			for (s = 0; s < seg->area_count; s++) {
-				if (seg->area[s].type != AREA_PV)
+				if (seg_type(seg, s) != AREA_PV)
 					continue;
 
 				/* FIXME Also check for segs on deleted LVs */
 
-				pv = seg->area[s].u.pv.pvseg->pv;
+				pv = seg_pv(seg, s);
 				if (!pv || !pv->dev) {
 					if (!_remove_lv(cmd, lv, &list_unsafe)) {
 						stack;
